#!/usr/bin/perl -w

###############################################################################
#
#     Proxy.pm
#  
#  Description: 
#
#     blaaaah blaaah....
#
#  Usage:
#
#      Proxy.pm [<args>]
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)
#  for more details
#
#  Author        : Dirk Barnikel <dirk.barnikel@gmx.de>
#  Created       : Wed Nov 24 18:04:20 1999 (scratch.barny.net)
#  Last Modified : Wed Nov 24 18:04:20 1999 by Dirk Barnikel
#  Revision      : 0.01
#  
#  CHANGES:
#
#  Wed Nov 24 18:04:20 1999 (Dirk Barnikel):
#
#    * Proxy.pm 0.01 created.
#
###############################################################################

package Proxy;

use Proxy::Config;
#use Proxy::Admin;
use Proxy::Callback;
use Proxy::Log;
use Proxy::Connection;

use IO::Socket;
use Socket;
use strict;
use vars qw($VERSION $DEBUG);

$Proxy::VERSION = 0.01;
$Proxy::DEBUG   = 0;



sub InitServer
  {

   

    die "Can't read configuration from $Proxy::Config::ConfigFile\n" 
      if( ! -r $Proxy::Config::ConfigFile );
    
    print "Loading Configuration from $Proxy::Config::ConfigFile...\n";
    &Proxy::Config::loadConfigfile();
    print "----- aPROXY $Proxy::Config::__VERSION at $Proxy::Config::Hostname -----\n";
    
    if( -r $Proxy::Config::RulesFile ){
      print "Loading Forward Rules from $Proxy::Config::RulesFile...\n";
      &Proxy::Config::loadRulesfile()
    }
    
    #use Data::Dumper; print Dumper([%Proxy::Config::ForwardRules] , ["Proxy::Config::ForwardRules"]);
    
    if($Proxy::Config::RunInBackground){ sleep(1) && exit if fork() }
    
    ## admin-port (only if configured)

    if(defined($Proxy::Config::AdminPort) && defined($Proxy::Config::AdminPass)) 
      {
	eval "use Proxy::Admin;";
	die $@ if $@;
	&Proxy::Admin::Console::SetupAdminSocket($Proxy::Config::AdminPort);	  
      }
    
   # ## monitor-port( only if configured)
#    &Proxy::Admin::Monitor::SetupMonitorSocket($Proxy::Config::MonitorPort)
#      if(defined($Proxy::Config::MonitorPort) && defined($Proxy::Config::MonitorPass));
    
    ## check-port
    &check_proxy_setup_socket( @Proxy::Config::__IO_SOCKETOPTS,  LocalPort => $Proxy::Config::CheckPort);
    
    ## setup all listening sockets    
    my ($port , $host_ref);
    while(($port,$host_ref) = each %Proxy::Config::ForwardRules )
      {
	my $service = getservbyport($$host_ref[1],"tcp")|| $$host_ref[1];
	
	&log_event("Forwarding $Proxy::Config::Hostname:$port =>",
		   $$host_ref[0] , "[$service]"	, );	    
	
	&Proxy::Connection::setup_listening_socket( \&Proxy::Connection::accept_connection ,
						    @Proxy::Config::__IO_SOCKETOPTS,
						    LocalPort => $port, );
      }
    
    &Proxy::Log::open_log($Proxy::Config::LogFile) if( $Proxy::Config::UseLog == 1 
						       &&
						       -w $Proxy::Config::LogFile 
						     );
      

    &write_pid_file();

    &log_event("Ready for connections!");    
    
    Proxy::Callback->event_loop();
  }

#----------------------------------------------------------------------------

sub usage      { die "usage: Proxy->InitServer()" }

#----------------------------------------------------------------------------


sub write_pid_file
  {
    open(PID , ">" . $Proxy::Config::PidFile) || die "open pid-file failed: $!\n";
    print PID $$ ,"\n" ;
    close(PID);
  }


#--------------------------------------------------------------------------

sub check_proxy_setup_socket
  {
    #print join(" " , @_ ) , "\n";
    my(%params) = @_ ;
    my($sock);

    if( $sock = IO::Socket::INET->new(%params) )
      {
	&log_event("Port $Proxy::Config::CheckPort opened for checking connections");
	Proxy::Callback->set_event_handler( $sock , "read" => \&check_proxy_do_check );
	return $sock;
      }
    else
      {
	&log_event("Can't bind port $Proxy::Config::CheckPort: $!");
	return undef;
      }
  }

sub check_proxy_do_check
  {
    my($sock) = shift ;
    my $handle = $sock->accept();
    $handle->print("READY\012\015");
    $handle->close();
  }

1;

__END__


