#!/usr/bin/perl -w

###############################################################################
#
#     Proxy::Admin.pm
#  
#  Description: 
#
#     blaaaah blaaah....
#
#  Usage:
#
#      Proxy::Admin.pm [<args>]
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)
#  for more details
#
#  Author        : Dirk Barnikel <dirk.barnikel@gmx.de>
#  Created       : Wed Nov 24 01:14:42 1999 (scratch.barny.net)
#  Last Modified : Wed Nov 24 01:14:42 1999 by Dirk Barnikel
#  Revision      : 0.01
#  
#  CHANGES:
#
#  Wed Nov 24 01:14:42 1999 (Dirk Barnikel):
#
#    * Proxy::Admin.pm 0.01 created.
#
###############################################################################

package Proxy::Admin;
use strict;

use Proxy::Callback;
use Proxy::Log;

&log_event("Loading Admin-Module.");

use IO::Socket;

use vars qw($VERSION $DEBUG);

$Proxy::Admin::VERSION = 0.01;
$Proxy::Admin::DEBUG   = 0;

############################################################
#
package Proxy::Admin::Console;
#
#############################################################

use vars qw(%Handles %OP);
use Proxy::Log;
use Proxy::Connection;
use Data::Dumper;
%OP = (
       'ADD'        => \&__op_add_forward_rule,
       'DEL'        => \&__op_del_forward_rule,
       'LIST'       => \&__op_rulelist,
       'SAVE'       => \&__op_save_forward_rules,
       'FWD'        => \&__op_forward,
       'TIMEOUT'    => \&__op_timeout,
       'STAT'       => \&__op_stat,       
       'EXIT'       => sub { __close(shift()) ; 1 },
       'HELP'       => sub { shift()->print("+OK Commands: " , join(" " , sort keys %OP) , "\012\015") },
       'KILL'       => sub { shift()->print("+OK Shutting down...\012\015") ; 
			     &log_event("Shutdown invoked by Administrator!");
			     sleep(1) ;
			     kill 9 => 0;
			   },
      );

sub __op_save_forward_rules
  {
    my($sock) = shift ;
    my($file);
    
    if( $file = &Proxy::Config::saveRulesfile() )
      {
	$sock->print("+OK Rules saved to $file\012\015");
      }
    else
      {
	$sock->print("-ERR Can't save rules\012\015");
      }
  }

sub __op_timeout
  {
    my($sock)    = shift ;
    my($timeout);
    
    if($timeout = shift()) 
      {      
	$Proxy::Config::Timeout = $timeout;
	$sock->print("+OK Connection timeout set to $timeout sec.\012\015");
      }
    else
      {
	$sock->print("+OK Connection timeout is $Proxy::Config::Timeout sec.\012\015");
      }
  }

sub __op_stat
  {
    my($sock) = shift;
    $sock->print("Local Time             : " , scalar localtime() , "\012\015");
    $sock->print("Requests since startup : $Proxy::Connection::NumConnections\012\015" );
    $sock->print("Active Connections     : " , scalar keys %Proxy::Connection::CONN , "\012\015" );
    $sock->print("+OK\012\015");
  }

sub __op_rulelist
  {
    my $sock = shift ;
    my($port,$host_ref);
    
    $sock->print(("-" x 60) , "\012\015" );
    $sock->print(" PORT   | TARGET\012\015", ("-" x 60), "\012\015");
    
    while( ($port,$host_ref) = each %Proxy::Config::ForwardRules ) 
      {
	my $service = getservbyport($$host_ref[1],"tcp") || $$host_ref[1];
	$sock->print(' ' ,__fstr($port,7), "| " , $$host_ref[0] ," [$service]\012\015");
      }
    $sock->print(("-" x 60) , "\012\015" , "+OK" , "\012\015" );
    1;
  }

sub __check_port_range
  {
    my $port = shift ;
    return ( $port > 0 && $port < 65536 ) ? 1 : 0 ;
  }


sub __op_add_forward_rule
  {
    my $sock = shift ;
    my( $localport, $host, $port ) = @_ ;
    
    return $sock->print("-ERR usage: ADD <local-port> <target-host> <target-port>\012\015")      
      unless( 
	     defined($localport) 
	     && defined($host) 
	     && defined($port) 
	     && $localport =~ /^\d+$/ 
	     && $port      =~ /^\d+$/
	     && __check_port_range($localport)
	     && __check_port_range($port)
	    );    
    
    if(&Proxy::Connection::setup_listening_socket
       (
	\&Proxy::Connection::accept_connection,
	@Proxy::Config::__IO_SOCKETOPTS ,
	LocalPort => $localport,
       )
      )
      {
	$Proxy::Config::ForwardRules{$localport} = [ $host , $port ];
	return $sock->print("+OK Forwarding $Proxy::Config::Hostname:$localport => $host:$port\012\015");
      }
    else
      {
	return $sock->print("-ERR$Proxy::Log::LastMessage\012\015");
      }
  }

sub __op_del_forward_rule
  {
    my($sock) = shift;
    my($port) = shift;

    return $sock->print("-ERR usage: DEL <port-number>\012\015")
      unless( $port =~ /^\d+$/ );

    return $sock->print("-ERR Not listening on port $port\012\015")
      unless( exists $Proxy::Config::ForwardRules{$port} );

    if( &Proxy::Connection::delete_listening_socket($port))
      {
	$sock->print("+OK Deleted forward rule for port $port\012\015");
      }
    else
      {
	$sock->print("+ERR Can't delete forward rule for $port\012\015");
      }
  }

sub __op_forward
  { 
    my $sock = shift();
    my( $host,$port ) = @_ ;
    my $target;
    
    return $sock->print("-ERR usage: FWD <target-host> <target-port>\012\015")      
      unless( defined($host) && defined($port) );    
   
    $sock->print("Trying...\012\015");
    if($target = &Proxy::Connection::connect_peerhost($host,$port))
      {
	$sock->print("Connected to $host:$port\012\015");
	
	###### ugly hack :-((((((
	#########################

	Proxy::Callback->set_event_handler 
	  (
	   $sock, 
	   "read" =>  sub { &Proxy::Connection::forward($sock,$target) }
	  );
	Proxy::Callback->set_event_handler
	  (
	   $target,
	   "read" => sub { &Proxy::Connection::forward($target,$sock) }
	  );		
	&log_event("Forwarding Admin Connection to $host:$port");
      }
    else
      {
	$sock->print("Connecting $host:$port failed!\012\015");
      }
  }					   

sub SetupAdminSocket
  {
    my($port) = shift ;
    my($sock) = IO::Socket::INET->new( @Proxy::Config::__IO_SOCKETOPTS , LocalPort => $port)
      || return &log_event("SetupAdminSocket() failed: $!");    
    &log_event("Port $Proxy::Config::AdminPort opened for admin connections");
    Proxy::Callback->set_event_handler($sock , "read" => \&__accept_admin_connection);      
  }

sub __accept_admin_connection
  {
    my($sock) = shift()->accept() ;
    $sock->autoflush(1);
    &log_event("Admin Connection requested");
    Proxy::Callback->set_event_handler($sock, "read" => \&__read_passwd );   
  }   

sub __read_passwd
  {
    my($sock) = shift;
    return __close($sock) 
      unless( substr($sock->getline(),0 ,-2) eq $Proxy::Config::AdminPass );
    $Handles{$sock} = $sock;
    $sock->print("aPROXY V.$Proxy::Config::__VERSION Admin Console on $Proxy::Config::Hostname\012\015");
    &log_event("Password for Admin Connection OK");
    Proxy::Callback->set_event_handler($sock, "read" => \&__parse_command ); 
  }

sub __read_line
  {
    my($sock) = shift;
    my $buf   = "";
    if(sysread($sock,$buf,512))
      {
	#print "__read_line (buffer) : $buf";
	return substr($buf,0,-2);
      }
    else
      {
	__close($sock);	
      }
  }

sub __parse_command
  {
    my $sock = shift ;
    my $command = __read_line($sock) 
      || return $sock->print("-ERR Unknown command\012\015");
    
    my @tokens = split( /\s+/ , $command ) ;
    my $op     = uc(shift(@tokens));

    if( exists $OP{$op} )
      {
	&{$OP{$op}}($sock, @tokens);
      }
    else
      {
	$sock->print("-ERR Unknown command\012\015");	
      }
  }

sub __close
  {
    my($sock) = shift ;
    Proxy::Callback->set_event_handler($sock , "read" => undef );
    close($sock);
    delete $Handles{$sock};
    &log_event("Admin Connection closed");
    1;
  }

sub __fstr      # returns a fix length string
  { 
    my $string = shift || "0" ;
    my $o_len  = shift || 25  ;
    my $sepr   = shift || " " ;
    my $i_len  = length($string);
    if( $i_len >= $o_len ) {
      return substr $string , 0 , $o_len ;
    }
    return $string . ( $sepr x ( $o_len - $i_len ));
  }


############################################################
#
#package Proxy::Admin::Monitor;
##
##############################################################

#use vars qw( $MonitorSocks %Handles ) ;
#use Proxy::Log;


#sub SetupMonitorSocket
#  {
#    my($port) = shift ;
#    my($sock) = IO::Socket::INET->new( @Proxy::Config::__IO_SOCKETOPTS , LocalPort => $port)
#      || return &log_event("SetupMonitorSocket() failed: $!");    
#    &log_event("Port $Proxy::Config::MonitorPort opened for monitoring connections");
#    Proxy::Callback->set_event_handler($sock , "read" => \&__accept_monitor_connection);        
#  }

#sub SendToMonitors
#  {
#    foreach my $handle( values %Handles)       
#      { 
#	$handle->print(@_ , "\012\015")
#	  || __close($handle);    
#      }
#  }



#sub __accept_monitor_connection
#  {
#    my($sock) = shift()->accept() ;
#    $sock->autoflush(1);
#    &log_event("Monitoring Connection requested");
#    Proxy::Callback->set_event_handler($sock, "read" => \&__read_passwd );   
#    $MonitorSocks++;
#  }   

#sub __read_passwd
#  {
#    my($sock) = shift;
#    return __close($sock) 
#      unless( substr($sock->getline(),0 ,-2) eq $Proxy::Config::MonitorPass );
#    $Handles{$sock} = $sock;
#    &log_event("Password for Monitoring Connection OK");
#    #$sock->print("$Proxy::Config::Hostname: Monitor Connection established\012\015");
#  }


#sub __close
#  {
#    my($sock) = shift ;
#    Proxy::Callback->set_event_handler($sock , "read" => undef );
#    $MonitorSocks--;
#    close($sock);
#    delete $Handles{$sock};
#    &log_event("Monitoring Connection closed");
#    1;
#  }

1;

__END__


