package Proxy::Callback;

use strict;
use IO::Select;
use IO::Socket;
use Carp;
use vars qw ($DEBUG %rd_callbacks %wt_callbacks $rd_handles $wt_handles);

$DEBUG = 0 ;

%rd_callbacks = ();
%wt_callbacks = ();
$rd_handles   = IO::Select->new();
$wt_handles   = IO::Select->new();

#--------------------
# Event loop routines 

sub set_event_handler 
  {
    shift unless ref($_[0]); # shift if first arg is package name
    my ($handle, %args) = @_;
    my $callback;
    if (exists $args{'write'}) 
      {
        $callback = $args{'write'};
        if ($callback) 
	  {
	    $wt_callbacks{$handle} = $callback;
	    $wt_handles->add($handle);
	  } 
	else 
	  {
            delete $wt_callbacks{$handle};
            $wt_handles->remove($handle);
	  }
      }
    if (exists $args{'read'}) 
      {
        $callback = $args{'read'};
        if ($callback) 
	  {
	    $rd_callbacks{$handle} = $callback;
            $rd_handles->add($handle);
	  } 
	else 
	  {
            delete $rd_callbacks{$handle};
            $rd_handles->remove($handle);
	  }
      }
  } 

sub event_loop 
  {
    my ($pkg, $loop_count) = @_; # event_loop(1) to process events once
    my ($conn, $r, $w, $rset, $wset);    
    while (1) 
      { 
	#eval
	#{
	    # Quit the loop if no handles left to process
	    last unless ( ($rd_handles->count()) || ($wt_handles->count()) );
	    ($rset, $wset) =
	      IO::Select->select ($rd_handles, $wt_handles, undef, undef);
	    
	    foreach $r (@$rset) 
	      {
		if( exists $rd_callbacks{$r} )
		  {
		    &{$rd_callbacks{$r}}($r) 
		  }
	      }
	    foreach $w (@$wset) 
	      {
		if( exists $wt_callbacks{$w} )
		  {	
		    &{$wt_callbacks{$w}}($w);
		  }
	      }
	    if (defined($loop_count)) 
	      {
		last unless --$loop_count;
	      }
	  #};
      } 
  }

1;

__END__

