#!/usr/bin/perl -w

###############################################################################
#
#     Proxy::Connection.pm
#  
#  Description: 
#
#     blaaaah blaaah....
#
#  Usage:
#
#      Proxy::Connection.pm [<args>]
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License (http://www.gnu.org/copyleft/gpl.html)
#  for more details
#
#  Author        : Dirk Barnikel <dirk.barnikel@gmx.de>
#  Created       : Wed Nov 24 00:32:35 1999 (scratch.barny.net)
#  Last Modified : Wed Nov 24 00:32:35 1999 by Dirk Barnikel
#  Revision      : 0.01
#  
#  CHANGES:
#
#  Wed Nov 24 00:32:35 1999 (Dirk Barnikel):
#
#    * Proxy::Connection.pm 0.01 created.
#
###############################################################################

package Proxy::Connection;

use strict;

use Proxy::Callback;
use Proxy::Log;

use IO::Socket;

use vars qw($VERSION $DEBUG);

$Proxy::Connection::VERSION = 0.01;
$Proxy::Connection::DEBUG   = 0;

use vars qw($main_sock %CONN %SUMMARY %HOSTCACHE %LISTENERS $NumConnections);


$NumConnections = "0" ;

#----------------------------------------------------------------------------

sub setup_listening_socket
  {
    my ($callback , %params) = @_ ;
    ## setup main socket
   
    my $sock ;
    my $port = $params{LocalPort} ;
    

    if( $sock = IO::Socket::INET->new( %params ) )
      {
	Proxy::Callback->set_event_handler( $sock , "read" => $callback);
	return $LISTENERS{$params{LocalPort}} = $sock ;
      }
    else 
      { 
	&log_event( "Can't bind port $params{LocalPort}: $!" ); 
	return undef;
      }
  }
#-----------------------------------------------------------------------------

sub delete_listening_socket
  {
    my($port) = shift;
    my $listener = delete $LISTENERS{$port};
    Proxy::Callback->set_event_handler( $listener, "read" => undef );
    close($listener);
    delete $Proxy::Config::ForwardRules{$port};
  }

#----------------------------------------------------------------------------

sub accept_connection
  {
    my $main_sock = shift;
    my $conn_sock ;
    
    if($conn_sock = $main_sock->accept())
      {
	$NumConnections++;
	$conn_sock->autoflush(1);
	my $shost = gethostbyaddr($conn_sock->peeraddr(), &AF_INET) || $conn_sock->peerhost();
	my $sport = $conn_sock->peerport();
	
	my ($thost, $tport) = @{$Proxy::Config::ForwardRules{$main_sock->sockport()}};

	&log_event("new connection from $shost:$sport");
	
	$SUMMARY{$conn_sock} = 0 ;
	$CONN{$conn_sock} = { 
			     S_SOCK  => $conn_sock,
			     S_HOST  => $shost, 
			     S_PORT  => $sport,
			     T_SOCK  => undef,
			     T_HOST  => $thost, 
			     T_PORT  => $tport,
			    };
	# connect to target-host
	my $target_sock;
	
	if( $target_sock = &connect_peerhost( $thost, $tport ))
	  {	   
	    $CONN{$conn_sock}->{T_SOCK} = $target_sock;
	    &log_event("connected: ", &hoststring($conn_sock) ," <=> ", &hoststring($target_sock));	
	    Proxy::Callback->set_event_handler( $conn_sock   , "read" => \&read_event );
	    Proxy::Callback->set_event_handler( $target_sock , "read" => \&read_event );
	    ### only if Admin Module is loaded ###
	  }
	else
	  {
	    delete $CONN{$conn_sock};
	    close($conn_sock) 
	  }	
      }
    1;
  }

#----------------------------------------------------------------------------

sub read_event
  {
    my $sock      = shift ;
    my $fwd_bytes = 0; 

    if( exists $CONN{$sock} ) # this is a source port
      {
	$fwd_bytes = &Proxy::Connection::forward( $sock, &get_target_by_source($sock) );
      }
    else                      # this is a target port
      {
	$fwd_bytes = &Proxy::Connection::forward( $sock, &get_source_by_target($sock) );
      }
    $SUMMARY{$sock} += $fwd_bytes ;
  }

#----------------------------------------------------------------------------

sub connect_peerhost
  {
    ## returns a opened socket
    
    my ($thost, $tport ) = @_;
    my (%params)         = ( PeerAddr => $thost, 
			     PeerPort => $tport,
			     Proto    => 'tcp' ,			     
			     Timeout  => $Proxy::Config::Timeout,
			   );
    
    my $tsock = IO::Socket::INET->new();
    
    {
      local($SIG{ALRM}) = sub { die "Timeout after $Proxy::Config::Timeout sec." };
      my $peeraddr = sockaddr_in( $params{PeerPort} ,inet_aton($params{PeerAddr}));      
      
      eval 
	{
	  socket( $tsock , &PF_INET , &SOCK_STREAM , getprotobyname( $params{Proto} ) )
	    or die "socket() failed!\n";
	  
	  alarm($Proxy::Config::Timeout);
	  
	  connect( $tsock , $peeraddr )
	    or die "$!\n";
	  
	  
	};   
      alarm(0);
    }
    
    if($@)
      {	
	$@ =~ s/(.+) at.+$/$1/gio;
	chomp $@;
	&log_event("can't establish connection to $thost:$tport ($@)");
	return undef;
      } 
    else
      {	
       	&log_event("outgoing connection established to $thost:$tport" );
	$SUMMARY{$tsock} = 0;
	return $tsock;
      }
  }

#----------------------------------------------------------------------------

sub forward 
  {
    my( $source_sock, $target_sock, $keep_source ) = @_;
    my( $in, $out ) = (0,0);
    my( $in_name , $out_name ) = ( &hoststring($source_sock), &hoststring($target_sock) ) ;
    my( $buf ) = "";
    my( $block_size ) = $Proxy::Config::__IO_BLOCKSIZE ;
    ## read data from $source_sock and write to $target_sock
    if( 
       ($in = sysread $source_sock , $buf , $block_size ) > 0 
       &&
       ($out = syswrite $target_sock , $buf , $in) > 0 
      )
      {
	$SUMMARY{$source_sock} += $in;
	$SUMMARY{$target_sock} += $out;
	return( $out );
      }
    ## close connection, cleanup eventhandlers
    else
      {	       	
	&log_event("Connection closed [" , $in_name , "/" , $out_name , "]");
	
	my @sockets_to_delete = ($target_sock,$source_sock);
	# close the sourcesocket unless forward if called with a third
	# argument ( example: forward($src,$dest,1);
	
	#shift @sockets_to_delete, $source_sock if( ! $keep_source );
	  
	for(@sockets_to_delete ) {	
	  #########################################################################
	  if( exists $CONN{$_}      ) { delete $CONN{$_}      }
	  if( exists $SUMMARY{$_}   ) { delete $SUMMARY{$_}   }
	  if( exists $HOSTCACHE{$_} ) { delete $HOSTCACHE{$_} }
	  #######################################################################
	  Proxy::Callback->set_event_handler( $_ , "read" => undef , "write" => undef );
	  close($_);	
	}
	return undef;
      }
    return ( $out > 0 ) ? $out : 0  ;      
  }

#---------------------------------------------------------------------------------

sub hoststring
  {
    # returns an array with hostname and port of the socket $sock
    my $sock = shift ;
    ############################################
    ###########################################
    unless( exists $HOSTCACHE{$sock} ){ 
      my $peername = gethostbyaddr($sock->peeraddr(),&AF_INET) || $sock->peerhost;
      $HOSTCACHE{$sock} = [ $peername , $sock->peerport ];
    }
    my( $host, $port ) = @{$HOSTCACHE{$sock}} ;
    return "$host:$port";
  }


#----------------------------------------------------------------------------
   
sub get_target_by_source
  {
    my $sock = shift;
    return $CONN{$sock}->{T_SOCK} || undef; 
  }

#----------------------------------------------------------------------------

sub get_source_by_target
  {
    my $sock = shift ;
    foreach( values %CONN )
      {	return $_->{S_SOCK} if( $_->{T_SOCK} == $sock ) 
      }

    return undef; # should nevener happen
  }

1;
__END__


